/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "xmem.h"
#include "buffer.h"
#include "queue.h"
#include "box.h"

// ***************************************************************************
// 
// ***************************************************************************
box *box_new(int id, int fd, netpeer *peer)
{
	box *b;
	
	b = xmalloc(sizeof(box));
	
	b->fd = fd;
	b->id = id;
	b->peer = *peer;
	b->host = xstrdup(peer->host);
	b->cores = 0;
	b->flags = 0;
	b->refs = 0;
	b->ack_h = -1;
	b->ack_n = 0;
	
	buffer_init(&b->inbuff, 0);
	buffer_init(&b->outbuff, 0);
	
	queue_init(&b->running);
	
	b->prev = NULL;
	b->next = NULL;
	
	return(b);
}


// ***************************************************************************
// 
// ***************************************************************************
void box_free(box *b)
{
	xfree(b->host);
	
	buffer_clear(&b->inbuff);
	buffer_clear(&b->outbuff);
	queue_clear(&b->running);
}

// ***************************************************************************
// 
// ***************************************************************************
void box_setup(box *b, char *host, int cores)
{
	if (b->host)
		xfree(b->host);
	
	b->host = xstrdup(host);
	b->cores = cores;
}


// ***************************************************************************
// 
// ***************************************************************************
void boxset_init(boxset *bs)
{
	bs->head = NULL;
	bs->tail = NULL;
	bs->n = 0;
}

void boxset_clear(boxset *bs)
{
	box *b, *c;
	
	b = bs->head;
	while (b) {
		c = b->next;
		
		box_free(b);
		
		b = c;
	}
	
	bs->head = NULL;
	bs->tail = NULL;
	bs->n = 0;
}

// ***************************************************************************
// 
// ***************************************************************************
void boxset_attach(boxset *bs, box *b)
{
	b->prev = bs->tail;
	b->next = NULL;
	
	if (bs->tail)
		bs->tail->next = b;
	
	bs->tail = b;
	
	if (!bs->head)
		bs->head = b;
	
	bs->n++;
}

void boxset_detach(boxset *bs, box *b)
{
	if (b->prev)
		b->prev->next = b->next;
	
	if (b->next)
		b->next->prev = b->prev;
	
	if (bs->head == b)
		bs->head = b->next;
	
	if (bs->tail == b)
		bs->tail = b->prev;
	
	b->prev = NULL;
	b->next = NULL;
	
	bs->n--;
}

// ***************************************************************************
// 
// ***************************************************************************
box *boxset_find(boxset *bs, int id)
{
	box *b = bs->head;
	
	while (b) {
		if (b->id == id)
			return(b);
		
		b = b->next;
	}
	
	return(NULL);
}


