/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TASK_BOX_H
#define TASK_BOX_H
#include "network.h"
#include "queue.h"

#define BOX_KICK	1
#define BOX_SYNC	2

typedef struct box_t {
	int fd, id;
	netpeer peer;
	char *host;
	int cores, flags;
	int refs;
	
	int ack_h;
	size_t ack_n;
	
	buffer inbuff;
	buffer outbuff;
	
	queue running;

	struct box_t *prev;
	struct box_t *next;
} box;

box *box_new(int id, int fd, netpeer *peer);
void box_free(box *b);

void box_setup(box *b, char *host, int cores);


typedef struct boxset_t {
	box *head;
	box *tail;
	int n;
} boxset;

void boxset_init(boxset *bs);
void boxset_clear(boxset *bs);

void boxset_attach(boxset *bs, box *b);
void boxset_detach(boxset *bs, box *b);

box *boxset_find(boxset *bs, int id);

#endif

