/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TASK_BUFFER_H
#define TASK_BUFFER_H
#include <stdarg.h>
#include <unistd.h>

typedef struct buffer_t {
	size_t mask, size, alloc, partial;
	void *data;
	int eof, error;
} buffer;


void buffer_init(buffer *b, size_t gran);
void buffer_clear(buffer *b);
void buffer_reset(buffer *b);

void buffer_alloc(buffer *b, size_t size);
void *buffer_get(buffer *b, size_t size);
void buffer_memcpy(buffer *b, const void *src, size_t size);
void buffer_memappend(buffer *b, const void *src, size_t size);

char *buffer_str(buffer *b);
void buffer_strcpy(buffer *b, const char *src);
void buffer_strappend(buffer *b, const char *src);
char *buffer_fappend(buffer *b, const char *format, ...);

int buffer_load(buffer *b, int fd, size_t max_size);
int buffer_save(buffer *b, int fd);
int buffer_load_path(buffer *b, const char *path, size_t max_size);
int buffer_save_path(buffer *b, const char *path);

void buffer_io_reset(buffer *b);
int buffer_io_read(buffer *b, int fd);
int buffer_io_write(buffer *b, int fd);
int buffer_io_complete(buffer *b);
int buffer_io_eof(buffer *b);
int buffer_io_error(buffer *b);

#endif

