/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#define _BSD_SOURCE
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include "config.h"

task_config conf;

static char config_hostname[1025];

void config_init(char **envp)
{
	int t;
	
	conf.envp = envp;
	conf.shell = NULL;
	conf.host = NULL;
	conf.file_size_max = TASK_DEFAULT_FILE_SIZE_MAX;
	conf.proto_buff_max = TASK_DEFAULT_PROTO_BUFF_MAX;
	conf.block_size = TASK_DEFAULT_BLOCK_SIZE;
	conf.default_service = TASK_DEFAULT_SERVICE;
	
	conf.cpus_online = sysconf(_SC_NPROCESSORS_ONLN);
	if ((conf.cpus_online <= 0) || (conf.cpus_online > 65536))
		conf.cpus_online = 1;
	
	t = 0;
	while (envp[t]) {
		if (!strncmp(envp[t], "SHELL=", 6)) {
			conf.shell = envp[t] + 6;
		} else if (!strncmp(envp[t], "HOSTNAME=", 9)) {
			conf.host = envp[t] + 9;
		}
		t++;
	}
	
	if (!conf.shell)
		conf.shell = TASK_DEFAULT_SHELL;
	if (!conf.host) {
		if (gethostname(config_hostname, sizeof(config_hostname))) {
			fprintf(stderr, "gethostname(): %s\n", strerror(errno));
			conf.host = TASK_DEFAULT_HOST;
		} else {
			conf.host = config_hostname;
		}
	}
}

void config_clear()
{
}


