/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TASK_QUEUE_H
#define TASK_QUEUE_H
#include <time.h>

typedef struct task_t {
	int id;
	
	char *dir;
	char *cmd;
	char *name;
	
	int restrict_box;
	int forward_box;
	
	char *outpath, *errpath;
	int outfd, errfd;
	
	time_t start_time;
	time_t stop_time;
	
	struct task_t *prev;
	struct task_t *next;
} task;


task *task_new(int id, char *dir, char *cmd, char *name);
void task_free(task *t);

int task_open(task *t);
void task_close(task *t, int rmfiles);

int task_matches(task *t, char *dir, char *name);

typedef struct queue_t {
	task *head;
	task *tail;
	int size;
} queue;

void queue_init(queue *q);
void queue_clear(queue *q);

void queue_prepend(queue *q, task *t);
void queue_append(queue *q, task *t);
void queue_unlink(queue *q, task *t);

task *queue_find(queue *q, int id);

#endif

