/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#define _POSIX_SOURCE
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "config.h"
#include "shell.h"

// **************************************************************************
//
// **************************************************************************
int exec_fork(exec_unit *u, char *path)
{
	int pin[2], pout[2], perr[2];
	char *argv[2];
	
	if (pipe(pin)) {
		fprintf(stderr, "exec_fork(): pipe(): %s\n", strerror(errno));
		return(-1);
	}

	if (pipe(pout)) {
		fprintf(stderr, "exec_fork(): pipe(): %s\n", strerror(errno));
		close(pin[0]);
		close(pin[1]);
		return(-1);
	}

	if (pipe(perr)) {
		fprintf(stderr, "exec_fork(): pipe(): %s\n", strerror(errno));
		close(pin[0]);
		close(pin[1]);
		close(pout[0]);
		close(pout[1]);
		return(-1);
	}

	u->pid = fork();
	
	if (u->pid == -1) {
		fprintf(stderr, "exec_fork(): fork(): %s\n", strerror(errno));
		close(pin[0]);
		close(pin[1]);
		close(pout[0]);
		close(pout[1]);
		close(perr[0]);
		close(perr[1]);
		return(-1);
	} else if (u->pid != 0) {
		u->infd = pin[1];
		u->outfd = pout[0];
		u->errfd = perr[0];
		close(pin[0]);
		close(pout[1]);
		close(perr[1]);
		return(0);
	}
	
	close(pin[1]);
	close(pout[0]);
	close(perr[0]);

	/*{
		char buff[32];
		
		fprintf(stderr, "Reading command\n");
		read(pin[0], buff, 32);
		fprintf(stderr, "Writing output\n");
		write(pout[1], "Okay\n", 5);
		fprintf(stderr, "Exiting\n");
		exit(0);
	}*/
	close(0);
	if (dup2(pin[0], 0) == -1) {
		fprintf(stderr, "dup2(): %s\n", strerror(errno));
		exit(1);
	}
	
	close(1);
	if (dup2(pout[1], 1) == -1) {
		fprintf(stderr, "dup2(): %s\n", strerror(errno));
		exit(1);
	}
	
	close(2);
	if (dup2(perr[1], 2) == -1)
		exit(1);
	
	close(pin[0]);
	close(pout[1]);
	close(perr[1]);
	
	argv[0] = path;
	argv[1] = NULL;

	execve(path, argv, conf.envp);
	
	fprintf(stderr, "%s: %s\n", path, strerror(errno));
	
	exit(1);
}

// **************************************************************************
//
// **************************************************************************
int exec_terminate(exec_unit *u)
{
	pid_t wr;
	int status;
	
	if (u->pid < 0)
		return(0);
	
	if (u->infd >= 0)
		close(u->infd);
	if (u->outfd >= 0)
		close(u->outfd);
	if (u->errfd >= 0)
		close(u->errfd);

	u->infd = -1;
	u->outfd = -1;
	u->errfd = -1;
	
	if (kill(u->pid, SIGTERM)) {
		fprintf(stderr, "kill(): %s\n", strerror(errno));
	}
	
	while (1) {
		wr = waitpid(u->pid, &status, 0);
		
		if (wr == -1) {
			if (errno != EINTR) {
				fprintf(stderr, "waitpid(): %s\n",
					strerror(errno));
				return(-1);
			}
		} else if (wr == u->pid) {
			u->pid = -1;
			return(0);
		} else {
			fprintf(stderr, "waitpid(): returned pid %d != %d\n",
				wr, u->pid);
			return(-1);
		}
	}
}


// **************************************************************************
//
// **************************************************************************



