/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TASK_UNITS_H
#define TASK_UNITS_H
#include "buffer.h"
#include "shell.h"

#define UNIT_END	1
#define UNIT_WATCH	2

typedef struct unit_t {
	int id;
	
	exec_unit exe;
	int flags;
	
	buffer inbuff;
	buffer outbuff;
	buffer errbuff;
	
	struct unit_t *prev;
	struct unit_t *next;
} unit;


unit *unit_new(int id, char *shell, char *cmd);
void unit_free(unit *u);

typedef struct unit_list_t {
	unit *head;
	unit *tail;
	int size;
} unit_list;

void ul_init(unit_list *l);
void ul_clear(unit_list *l);

void ul_prepend(unit_list *l, unit *t);
void ul_append(unit_list *l, unit *t);
void ul_unlink(unit_list *l, unit *t);

unit *ul_find(unit_list *l, int id, int flags);

#endif

