/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "xmem.h"


// ***************************************************************************
// 
// ***************************************************************************
void *dxmalloc(size_t n, const char *what,
	const char *func, const char *file, int line)
{
	void *r;
	
	r = malloc(n);
	
	if (!r) {
		fprintf(stderr, "%s:%d: %s(): malloc(%s): "
			"failed allocating %zd bytes\n",
			file, line, func, what, n);
		exit(1);
	}
	
	return(r);
}


// ***************************************************************************
// 
// ***************************************************************************
void *dxrealloc(void *p, size_t n, const char *pname, const char *what,
	const char *func, const char *file, int line)
{
	void *r;
	
	r = realloc(p, n);
	
	if (!r) {
		fprintf(stderr, "%s:%d: %s(): realloc(%s, %s): "
			"failed allocating %zd bytes\n",
			file, line, func, pname, what, n);
		exit(1);
	}
	
	return(r);
}


// ***************************************************************************
// 
// ***************************************************************************
void dxfree(void *p, const char *pname,
	const char *func, const char *file, int line)
{
	free(p);
}

// **************************************************************************
// 
// **************************************************************************
char *vxprintf(size_t *len_r, const char *format, va_list ap)
{
	int len;
	char *dst;
	va_list aq;
	
	va_copy(aq, ap);

	len = vsnprintf(NULL, 0, format, aq);
	
	va_end(aq);

	dst = malloc(len + 1);
	if (!dst) {
		fprintf(stderr, "varsnprintf(): malloc(%d) failed\n",
			len + 1);
		exit(1);
	}

	vsnprintf(dst, len + 1, format, ap);

	if (len_r)
		*len_r = len;
	
	return(dst);
}


// **************************************************************************
// 
// **************************************************************************
char *xprintf(size_t *len_r, const char *format, ...)
{
	va_list ap;
	char *r;
	
	va_start(ap, format);
	r = vxprintf(len_r, format, ap);
	va_end(ap);
	
	return(r);
}


