/*
    This file is part of task -- a task scheduler for distributed computations
    Copyright (C) 2012  Laurent Poirrier

    task is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TASK_XMEM_H
#define TASK_XMEM_H
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

// failsafe funcs
void *dxmalloc(size_t n, const char *what,
	const char *func, const char *file, int line);
void *dxrealloc(void *p, size_t n, const char *pname, const char *what,
	const char *func, const char *file, int line);
void dxfree(void *p, const char *pname,
	const char *func, const char *file, int line);

#define xmalloc(n)	dxmalloc(n, #n, __func__, __FILE__, __LINE__)
#define xrealloc(p, n)	dxrealloc(p, n, #p, #n, __func__, __FILE__, __LINE__)
#define xfree(p)	dxfree(p, #p, __func__, __FILE__, __LINE__)

#define xstrdup(s)	({	size_t __local_l = strlen(s) + 1; \
				char *__local_d = xmalloc(__local_l); \
				memcpy(__local_d, s, __local_l); __local_d; })

char *vxprintf(size_t *len_r, const char *format, va_list ap);
char *xprintf(size_t *len_r, const char *format, ...);



#endif

